@@ -30,14 +30,15 @@
 #include <time.h>
 
 typedef struct _ProcConfig ProcConfig;
-typedef struct _PrettyTable PrettyTable;
-typedef struct _ProcInfo ProcInfo;
-typedef struct _ProcData ProcData;
+struct ProcInfo;
+struct ProcData;
 
 #include "smooth_refresh.h"
 
 #include "load-graph.h"
 
+#include "prettytable.h"
+
 enum
 {
 	ALL_PROCESSES,
@@ -45,11 +46,9 @@ enum
 	ACTIVE_PROCESSES
 };
 
-enum
-{
-	MIN_UPDATE_INTERVAL = 1 * 1000,
-	MAX_UPDATE_INTERVAL = 100 * 1000
-};
+
+static const unsigned MIN_UPDATE_INTERVAL =   1 * 1000;
+static const unsigned MAX_UPDATE_INTERVAL = 100 * 1000;
 
 
 enum ProcmanTab
@@ -66,12 +65,11 @@ struct _ProcConfig
 	gint		width;
 	gint		height;
         gboolean	show_kill_warning;
-        gboolean	show_hide_message;
         gboolean	show_tree;
 	gboolean	show_all_fs;
-	guint		update_interval;
- 	gint		graph_update_interval;
- 	gint		disks_update_interval;
+	int		update_interval;
+ 	int		graph_update_interval;
+ 	int		disks_update_interval;
 	gint		whose_process;
 	gint		current_tab;
 	GdkColor	cpu_color[GLIBTOP_NCPU];
@@ -85,8 +83,16 @@ struct _ProcConfig
 };
 
 
-struct _ProcInfo
+class ProcInfo
 {
+	ProcInfo& operator=(const ProcInfo&);
+	ProcInfo(const ProcInfo&);
+ public:
+	ProcInfo(pid_t pid);
+	~ProcInfo();
+	// adds one more ref to icon
+	void set_icon(GdkPixbuf *icon);
+
 	GtkTreeIter	node;
 	GtkTreePath	*path;
 	ProcInfo	*parent;
@@ -104,11 +110,17 @@ struct _ProcInfo
 	time_t		start_time;
 	guint64		cpu_time_last;
 
-	guint64		vmsize;
-	guint64		memres;
-	guint64		memwritable;
-	guint64		memshared;
-	guint64		mem; /* estimated memory usage */
+	// all these members are filled with libgtop which uses
+	// guint64 (to have fixed size data) but we don't need more
+	// than an unsigned long (even for 32bit apps on a 64bit
+	// kernel) as these data are amounts, not offsets.
+	unsigned long	vmsize;
+	unsigned long	memres;
+	unsigned long	memwritable;
+	unsigned long	memshared;
+	unsigned long	mem; /* estimated memory usage */
+
+	// wnck gives an unsigned long
 	unsigned long	memxserver;
 
 	guint		pid;
@@ -119,11 +131,13 @@ struct _ProcInfo
 
 	guint		is_visible	: 1;
 	guint		is_running	: 1;
-	guint		is_blacklisted	: 1;
 };
 
-struct _ProcData
+struct ProcData
 {
+	// lazy initialization
+	static ProcData* get_instance();
+
 	GtkUIManager	*uimanager;
 	GtkActionGroup	*action_group;
 	GtkWidget	*statusbar;
@@ -170,9 +184,7 @@ struct _ProcData
 	GList		*info;
 	GHashTable	*pids;
 
-	PrettyTable	*pretty_table;
-	GList		*blacklist;
-	gint		blacklist_num;
+	PrettyTable	pretty_table;
 
 	GConfClient	*client;
 	GtkWidget	*app;
@@ -182,12 +194,7 @@ struct _ProcData
 	/* cached username */
 	GStringChunk	*users;
 
-
-	/* libgtop uses guint64 but we use a float because
-	   frequency is ~always == 100
-	   and because we display cpu_time as %.1f seconds
-	*/
-	float		frequency;
+	unsigned	frequency;
 
 	SmoothRefresh  *smooth_refresh;
 };
@@ -213,5 +220,4 @@ struct KillArgs
 	int signal;
 };
 
-
 #endif /* _PROCMAN_PROCMAN_H_ */
